"""FraiseQL version information.

This module contains the current version of FraiseQL. The version follows
semantic versioning (MAJOR.MINOR.PATCH) with optional pre-release identifiers.

Version format: MAJOR.MINOR.PATCH[-PRERELEASE][+BUILD]

Examples:
- 1.8.0 - Stable release
- 1.8.0-beta.1 - Beta pre-release
- 1.8.0-rc.1 - Release candidate
- 1.8.0+build.123 - Build metadata

This version is used by:
- Package installation and distribution
- Compatibility checking
- Feature flag activation
- Documentation versioning
"""

__version__ = "1.8.0-beta.1"
